var WidgetRegistry = {
  _widgetsById: [],
  _widgetsByType: [],
  
  add: function(widgetId, widgetType, widget) {
    WidgetRegistry._widgetsById[widgetId + ""] = widget;
    
    if (!WidgetRegistry._widgetsByType[widgetType]) {
      WidgetRegistry._widgetsByType[widgetType] = [];
    }
    WidgetRegistry._widgetsByType[widgetType].push(widget);
  }, 
  
  getWidgetById: function(widgetId) {
    return WidgetRegistry._widgetsById[widgetId + ""];
  }, 
  
  getWidgetsByType: function(widgetType) {
    return WidgetRegistry._widgetsByType[widgetType];
  }
}

var FriendsWidget = Class.create();
FriendsWidget.prototype = {
	
	options : { searchUrl:"/myhabbo/avatarlist/friendsearchpaging",ownerParameter:"&_mypage.requested.account=" },

	initialize : function(ownerId, widgetId) {
		this.searchString = "";
        this.pageNumber = 1;
        this.ownerId = ownerId;
		this.widgetId = widgetId;
		this.widgetEl = $("widget-" + widgetId);
        this.containerEl = $("avatarlist-content");
		this.listElement = $("avatar-list-list");
        this.pagingElement = $("avatar-list-paging");

        if (this.listElement) {
			this.containerEl.onclick = this._processClick.bindAsEventListener(this);
			this.infoEl = document.getElementsByClassName("avatar-list-info", this.widgetEl)[0];
			this.infoContentEl = document.getElementsByClassName("avatar-list-info-container", this.infoEl)[0];
			this.closeLink = document.getElementsByClassName("avatar-list-info-close", this.infoEl)[0];
			this.closeLink.onclick = this.hideAccountDetails.bindAsEventListener(this);
			var self = this;
            Event.observe("avatarlist-search-button", "click", function(e) { Event.stop(e); self._processSearch(e); });
            Event.observe("avatarlist-search-string", "keypress", function(e) {
                if (e.keyCode == Event.KEY_RETURN) {
                    self._processSearch(e);
                }
            });
        }
	}, 
	
	showAccountDetails : function(showId) {
		this.infoEl = $("avatar-list-info");
        this.listElement = $("avatar-list-list");
        this.pagingElement = $("avatar-list-paging");
        this.searchElement = $("avatar-list-search");
        Element.show(this.infoEl);
        Element.hide(this.listElement);
        Element.hide(this.pagingElement);
        Element.hide(this.searchElement);
        this.infoEl.style.display = "block";
        Element.wait(this.infoEl);
        this.showId = showId;
		var self = this;
		new Ajax.Request(habboReqPath + "/myhabbo/avatarlist/avatarinfo", {
			method:"post", parameters:this._getInfoParameters(), onComplete:function(req, json) {
				self.infoEl.innerHTML = req.responseText;
				
				var infoCloseLink = document.getElementsByClassName("avatar-list-info-close", self.infoEl)[0];
				infoCloseLink.onclick = self.hideAccountDetails.bindAsEventListener(self);
				
				self._addLinkObservers();
			}
		});
	}, 
	
	hideAccountDetails : function(e) {
		Event.stop(e);
		this.infoContentEl.innerHTML = "";
		Element.hide(this.infoEl);
		Element.show(this.listElement);
        Element.show(this.pagingElement);
        Element.show(this.searchElement);
	}, 
	
	leaveFromGroup : function(e) { Event.stop(e); }, 
	removeFromGroup : function(e) { Event.stop(e); }, 
	revokeRights : function(e) { Event.stop(e); }, 
	
	_parseAccountIdFromElementId : function(elementId) {
		return elementId.substring(elementId.lastIndexOf("-")+1);
	}, 
	
	_getInfoParameters : function() {
		return "ownerAccountId=" + encodeURIComponent(this.ownerId) + "&anAccountId=" + encodeURIComponent(this.showId);
	}, 

    _processClick : function(e) {
        var clickedElement = Event.element(e);
        if (clickedElement.nodeName.toLowerCase() == "a" && clickedElement.className == "avatar-list-open-link") {
            Event.stop(e);
            this._processOpenClick(e);
        } else if (clickedElement.nodeName.toLowerCase() == "a" && clickedElement.className == "avatar-list-paging-link") {
            Event.stop(e);
            this._processSearch(e);
        }
    },

    _processOpenClick : function(e) {
		var clickedEl = Event.element(e);
		if (clickedEl.nodeName.toLowerCase() == "a" && clickedEl.className == "avatar-list-open-link") {
			var id = this._parseAccountIdFromElementId(clickedEl.parentNode.parentNode.id);
			this.showAccountDetails(id);
		}
	}, 
	
	_addLinkObservers : function() {},

    _processSearch : function(e) {
		var clickedElement = Event.element(e);
        var pageNumber = parseInt($F("pageNumber"));
        var totalPages = parseInt($F("totalPages"));
        var targetPageNumber = 1;
        if (clickedElement.id == "avatarlist-search-first") {
            targetPageNumber = 1;
        } else if (clickedElement.id == "avatarlist-search-previous") {
            targetPageNumber = pageNumber - 1;
        } else if (clickedElement.id == "avatarlist-search-next") {
            targetPageNumber = pageNumber + 1;
        } else if (clickedElement.id == "avatarlist-search-last") {
            targetPageNumber = totalPages;
        } else if (clickedElement.parentNode.id == "avatarlist-search-button" || clickedElement.id == "avatarlist-search-string") {
            this.searchString = $F("avatarlist-search-string");
		    targetPageNumber = 1;
        }

        var self = this;                                                                                       ''
        new Ajax.Updater("avatarlist-content",
			habboReqPath + this.options.searchUrl,
			{ method: "post",
			parameters: "pageNumber=" + encodeURIComponent(targetPageNumber) +
			"&searchString=" + encodeURIComponent(this.searchString) +
			"&widgetId=" + this.widgetId +
			this.options.ownerParameter + this.ownerId
        });
    }

};

var MemberWidget = Class.extend(FriendsWidget, {
    options : {searchUrl:"/myhabbo/avatarlist/membersearchpaging",ownerParameter:"&_groupspage.requested.group=" }, 

	leaveFromGroup : function(e) {
		Event.stop(e);
		openGroupActionDialog("/groups/actions/confirm_leave", "/groups/actions/leave", this.showId, this.ownerId, this);
	}, 
	
	removeFromGroup : function(e) {
		Event.stop(e);
		openGroupActionDialog("/groups/actions/confirm_remove", "/groups/actions/remove", this.showId, this.ownerId, this);
	}, 
	
	giveRights : function(e) {
		Event.stop(e);
		openGroupActionDialog("/groups/actions/confirm_give_rights", "/groups/actions/give_rights", this.showId, this.ownerId, this);
	}, 
	
	revokeRights : function(e) {
		Event.stop(e);
		openGroupActionDialog("/groups/actions/confirm_revoke_rights", "/groups/actions/revoke_rights", this.showId, this.ownerId, this);
	}, 
	
	_getInfoParameters : function() {
		return "groupId=" + encodeURIComponent(this.ownerId) + "&anAccountId=" + encodeURIComponent(this.showId);
	}, 
	
	_addLinkObservers : function() {
		var self = this;
		document.getElementsByClassName("avatar-info-rights-leave", this.infoEl).each(function(el) {
			el.onclick = self.leaveFromGroup.bindAsEventListener(self);
		});
		document.getElementsByClassName("avatar-info-rights-remove", this.infoEl).each(function(el) {
			el.onclick = self.removeFromGroup.bindAsEventListener(self);
		});
		document.getElementsByClassName("avatar-info-rights-give", this.infoEl).each(function(el) {
			el.onclick = self.giveRights.bindAsEventListener(self);
		});
		document.getElementsByClassName("avatar-info-rights-revoke", this.infoEl).each(function(el) {
			el.onclick = self.revokeRights.bindAsEventListener(self);
		});
	}
});

var ScrollWatcher = Class.create();
ScrollWatcher.prototype = {

	initialize: function(widgetId, className, callBack) {
		this.widgetId = widgetId;
        this.className = className;
		this.callBack = callBack;
		this.noticed = [];
		this.listItems();
		this.lastUpdate = 0;
		this.lastScrollTop = 0;
		
		var self = this;
		new PeriodicalExecuter(function() { self.update(self); }, .8);
	}, 
	
	update: function(e) {
		this.widgetEl = $("widget-" + this.widgetId);
        this.scrollDiv = document.getElementsByClassName("avatar-widget-list-container", this.widgetEl)[0];
		this.scrollDivHeight = Element.getHeight(this.scrollDiv);
        if (this.scrollDiv.scrollTop != this.lastScrollTop) {
			this.listItems();
            var self = this;
			this.items.each(function(el) {
				if (el.offsetTop + Element.getHeight(el) >= self.scrollDiv.scrollTop && 
					el.offsetTop < self.scrollDiv.scrollTop + self.scrollDivHeight) {
					if (self.callBack) {
						self.callBack(el);
					}
					self.noticed.push(el);
					el.className = "";
				}
			});
			this.listItems();
		}
		this.lastScrollTop = this.scrollDiv.scrollTop;
	},
	
	listItems: function() {
		this.items = document.getElementsByClassName(this.className, this.scrollDiv);
	}

};

var UpdateQueue = Class.create();
UpdateQueue.prototype = {

	initialize: function() {
		this.queue = [];
	}, 
	
	push: function(obj) {
		this.queue.push(obj);
	},
	
	flush: function() {
		var values = this.queue;
		this.queue = [];
		
		return values;
		
	}

};

var GuestbookWidget = Class.create();
GuestbookWidget.prototype = {

	initialize : function(accountId, widgetId, maxMessageLength) {
		this.accountId = accountId;
		this.widgetId = widgetId;
		this.maxMessageLength = maxMessageLength;
		
		var self = this;
		
		if ($('guestbook-open-dialog')) {
			Event.observe('guestbook-open-dialog', "click", function(e) {
				Event.stop(e);
				var offsets = Position.cumulativeOffset($('guestbook-open-dialog'));
				var target = $('guestbook-form-dialog');
				Position.absolutize(target);
			    target.style.top    = offsets[1] + 'px';
			    target.style.left   = (offsets[0] - 80) + 'px';
				Element.hide($('guestbook-preview-tab'));
				Element.show($('guestbook-form-tab'));
			    Element.show('guestbook-form-dialog');
			    $('guestbook-message').value = '';
			    $('guestbook-form-preview').disabled = 'true';
			    Element.addClassName($('guestbook-form-preview'), "disabled");
			    Field.activate($('guestbook-message'));
			});
			
			var formCancelHandler = function(e) {
				Event.stop(e);
				Element.hide($('guestbook-form-dialog'));
			};
			
			Event.observe('guestbook-form-dialog-exit', 'click', formCancelHandler);
			Event.observe('guestbook-form-cancel', 'click', formCancelHandler);
			
			Event.observe('guestbook-form-preview', 'click', function(e) {
				Event.stop(e);
				if ('true' == $('guestbook-form-preview').disabled) {
					return;
				}
				if ($F('guestbook-message').length > 0) {
					Element.hide($('guestbook-form-tab'));
                    Element.wait($('guestbook-preview-tab'));
                    Element.show($('guestbook-preview-tab'));
					new Ajax.Updater($("guestbook-preview-tab"), habboReqPath + "/myhabbo/guestbook/preview", {
			        		method : "post",
			                parameters : Form.serialize($('guestbook-form')) + "&widgetId=" + encodeURIComponent(self.widgetId),
			                evalScripts : true,
			                onComplete : function() {
								Event.observe('guestbook-form-post', 'click', function(e) {
									Event.stop(e);
									new Ajax.Updater($("guestbook-entry-container"), habboReqPath + "/myhabbo/guestbook/add", {
							        		method : "post",
							                parameters : Form.serialize($('guestbook-form')) + "&widgetId=" + encodeURIComponent(self.widgetId),
							                evalScripts : true,
							                insertion : Insertion.Top,
							                onComplete : function(response, status) {
                                                if (status && status.spam == "true") {
							                		return;
							                	}
                                                if ($('guestbook-empty-notes')) {
							                		Element.hide('guestbook-empty-notes');
							                	}
                                                Element.hide('guestbook-form-dialog');
                                                $("guestbook-entry-container").scrollTop = 0;
                                                var firstEntry = $$('#guestbook-entry-container .guestbook-entry').first();
                                                new Effect.Pulsate(firstEntry,
													{ afterFinish : function() { Element.setOpacity(firstEntry, 1); } }
												);
                                                self.incrementSize();
                                            }
							            });
								});
								
								Event.observe('guestbook-form-continue', 'click', function(e) {
									Event.stop(e);
									Element.hide($('guestbook-preview-tab'));
									Element.show($('guestbook-form-tab'));
			    					Field.focus($('guestbook-message'));
								});
								
			                }
			            });
				}
			});
			
			new Form.Element.Observer($('guestbook-message'), .5, (function(e) {
				var currMessageLength = $F('guestbook-message').length;
				if (currMessageLength > 0 && currMessageLength <= this.maxMessageLength) {
					$('guestbook-form-preview').disabled = '';
					Element.removeClassName($('guestbook-form-preview'), "disabled");
				} else {
					$('guestbook-form-preview').disabled = 'true';
					Element.addClassName($('guestbook-form-preview'), "disabled");
				}
			}).bind(this));
		}
		if ($("guestbook-delete-dialog")) {
			var self = this;
			Event.observe($("guestbook-delete"), "click", function(e) { Event.stop(e); self.doRemoveEntry($('guestbook-delete-id').value); self.hideRemoveConfirmation(); });
			var cancelDelete = function(e) { Event.stop(e); self.hideRemoveConfirmation(); };
			Event.observe($("guestbook-delete-dialog-exit"), "click", cancelDelete);
			Event.observe($("guestbook-delete-cancel"), "click", cancelDelete);
		}
		
		Event.observe($('guestbook-entry-container'), 'click', this.handleActions.bindAsEventListener(this));
		this.monitorEventListener = this.monitorScrollPosition.bind(this);
	},
	
	handleActions : function(e) {
		var clickedEl = Event.element(e);
		if (clickedEl.className == 'gbentry-delete') {
			Event.stop(e);
			var entryId = clickedEl.id.substring('gbentry-delete-'.length);
			this.removeEntry(entryId); 
		}
		if (clickedEl.className == 'gbentry-report') {
			Event.stop(e);
			var entryId = clickedEl.id.substring('gbentry-report-'.length);				
			
			ReportDialogManager.show("guestbook", entryId, clickedEl, {setWidth: false, setHeight: false, offsetTop: 0,offsetLeft: -120});
		}
	},
		
	monitorScrollPosition : function() {
		var element = $('guestbook-entry-container');
		if (!$('gb-progress')) {
            var progressEntry = Builder.node("div", { id:"gb-progress", style:"margin: 20px 0 60px 0; visibility: hidden" });
            $('guestbook-entry-container').appendChild(progressEntry);
            Element.wait(progressEntry);
		}
		if (element.scrollTop > 0 && (element.scrollTop + element.clientHeight == element.scrollHeight)) {
			element.scrollTop = element.scrollHeight - element.clientHeight;
			var last_entry_id = $$('#guestbook-entry-container .guestbook-entry').last().id.substring('guestbook-entry-'.length);
			$('gb-progress').style.visibility="";
			var self = this;
			new Ajax.Updater($("guestbook-entry-container"), habboReqPath + "/myhabbo/guestbook/list", {
	        		method : "post",
	                parameters : "ownerId=" + self.accountId + "&lastEntryId=" + last_entry_id + "&widgetId=" + encodeURIComponent(self.widgetId),
	                evalScripts : true,
	                insertion : Insertion.Bottom,
	                onComplete: (function(response, status) {
	                	$('gb-progress').remove();
	                	if (status.lastPage == 'false') {
	                		setTimeout(this.monitorEventListener, 300);
	                	}
	                }).bind(this)
	            });
		} else {
			setTimeout(this.monitorEventListener, 300);
		}
	},
	
	removeEntry : function (id) {
		var offsets = Position.cumulativeOffset($("gbentry-delete-" + id));
		var target = $("guestbook-delete-dialog");
		Position.absolutize(target);
		var containerScrollTop = $('guestbook-entry-container').scrollTop;
	    target.style.top    = (offsets[1] - containerScrollTop) + 'px';
	    target.style.left   = (offsets[0]-250) + 'px';
	    Element.show(target);
	    $('guestbook-delete-id').value = id;
	},
	
	hideRemoveConfirmation : function() {
		$('guestbook-delete-id').value = "";
	    Element.hide($("guestbook-delete-dialog"));
	},
	
	doRemoveEntry : function (id) {
		new Ajax.Request(habboReqPath + "/myhabbo/guestbook/remove", {
			parameters: "entryId=" + encodeURIComponent(id) + "&widgetId=" + encodeURIComponent(this.widgetId),
			onSuccess: function(req) {
				new Effect.Fade($('guestbook-entry-' + id),
					{ afterFinish : function() { $('guestbook-entry-' + id).remove(); } }
				);
				self.decrementSize();
            }
		});
	},
	
	incrementSize : function() {
		var target = $('guestbook-size');
		if (target) {
			var newValue = parseInt(target.innerHTML) + 1;
			target.innerHTML = newValue;
		}
	},
	
	decrementSize : function() {
		var target = $('guestbook-size');
		if (target) {
			var newValue = parseInt(target.innerHTML) - 1;
			target.innerHTML = newValue;
		}	
	},
	
	updateOptionsList : function(optionValue) {
		var select = $('guestbook-privacy-options');
		$A(select.options).each(function(option) {
			if (option.value == optionValue) { select.selectedIndex = option.index; }
		});		
	}


};

var RatingObserver = Class.create();

RatingObserver.prototype = {
	initialize : function(commonAjaxParams, elementToObserve, urlToCall, innerHtmlParamName) {
        this.commonAjaxParams = commonAjaxParams;
        this.elementToObserve = elementToObserve;
        this.urlToCall = urlToCall;
        this.elementToObserve = elementToObserve;
        this.innerHtmlParamName = innerHtmlParamName;
        Event.observe(elementToObserve, "click", this.handleEvent.bindAsEventListener(this));
    },

    handleEvent : function(e) {
        var params;
        Event.stop(e);
        if(this.innerHtmlParamName != null){
            params = this.commonAjaxParams.parameters + "&" + this.innerHtmlParamName + "=" + $(this.elementToObserve).innerHTML;
        }
        else{
            params = this.commonAjaxParams.parameters
        }

        $(this.commonAjaxParams.elementToUpdate).innerHTML = ""; 
        new Ajax.Updater(this.commonAjaxParams.elementToUpdate, habboReqPath + this.urlToCall,{ 
            method : "get",
            parameters : params,
            evalScripts : false,
            insertion : Insertion.Bottom,
            onComplete : this.commonAjaxParams.onCompleteFunction
            
        })
    }

};

var CommonRatingObserverParams = Class.create();

CommonRatingObserverParams.prototype = {
    initialize : function( elementToUpdate, parameters, onCompleteFunction) {
        this.elementToUpdate = elementToUpdate;
        this.parameters = parameters;
        this.onCompleteFunction  = onCompleteFunction;
    }
};

var RatingWidget = Class.create();

RatingWidget.prototype = {
    initialize : function(ownerId, ratingId){
    
        this.idParams = "ownerId=" + ownerId + "&ratingId=" + ratingId + "&_mypage.requested.account="+ownerId;
        
        //html-elements
        this.mainDiv = "rating-main";
        this.givenRate = "rating-rate";
        this.resetLink = "ratings-reset-link";
        this.ratingStarClassName = "rater";
        
        this.commonObserverParams = new CommonRatingObserverParams( this.mainDiv, this.idParams, this.refreshObservers.bind(this));
            
        this.refreshObservers();
    },

    refreshObservers : function(){
           if($(this.resetLink) &&  this.resetLinkObserver == null){
               this.resetLinkObserver = new RatingObserver(this.commonObserverParams, this.resetLink  , "/myhabbo/rating/reset_ratings", null);
            }
           //rating star listeners
           var ratingStars = document.getElementsByClassName(this.ratingStarClassName);
           for(var i=0; i<ratingStars.length; ++i){
			  new RatingObserver(this.commonObserverParams, ratingStars[i]  , "/myhabbo/rating/rate", "givenRate");
		   }
    }

    
};


var GroupsWidget = Class.create();
GroupsWidget.prototype = {

	initialize : function(ownerId, widgetId) {
		this.ownerId = ownerId;
		this.widgetId = widgetId;
		this.widgetEl = $("widget-" + widgetId);
		this.containerEl = document.getElementsByClassName("groups-list-container", this.widgetEl)[0];
		this.listEl = document.getElementsByClassName("groups-list", this.containerEl)[0];
		this.loadingEl = document.getElementsByClassName("groups-list-loading", this.widgetEl)[0];
		this.infoEl = document.getElementsByClassName("groups-list-info", this.widgetEl)[0];
			
		this.closeLink = document.getElementsByClassName("groups-loading-close", this.loadingEl)[0];
		this.closeLink.onclick = this.hideGroupDetails.bindAsEventListener(this);
		
		this._addOpenEventHandlers();
		
	}, 
	
	showGroupDetails : function(e) {
		if (Event.element(e).nodeName.toLowerCase() == "a") {
			return;
		}
		
		Event.stop(e);
		var target = Event.findElement(e, "li");
		this.groupId = this._parseGroupIdFromElementId(target.id);
		GroupFavoriteSelector.init(this, this.ownerId, this.groupId, this.onCompleteCallback.bind(this));
		Element.hide(this.containerEl);
		this.loadingEl.style.display = "block";
		
		var self = this;
		new Ajax.Request(habboReqPath + "/myhabbo/groups/groupinfo", { method:"post", parameters:"ownerId="+encodeURIComponent(this.ownerId)+"&groupId="+encodeURIComponent(this.groupId), onComplete:function(req, json) {
				self.infoEl.innerHTML = req.responseText;
				
				Element.hide(self.loadingEl);
				self.infoEl.style.display = "block";
				
				self.closeLink = document.getElementsByClassName("groups-info-close", self.infoEl)[0];
				self.closeLink.onclick = self.hideGroupDetails.bindAsEventListener(self);
				
				document.getElementsByClassName("groups-info-select-favorite", self.infoEl).each(function(el) {
					el.onclick = GroupFavoriteSelector.selectFavorite.bindAsEventListener(self);
				});
				
				document.getElementsByClassName("groups-info-deselect-favorite", self.infoEl).each(function(el) {
					el.onclick = GroupFavoriteSelector.deselectFavorite.bindAsEventListener(self);
				});
			}
		});
	}, 
	
	hideGroupDetails : function(e) {
		if (e) { Event.stop(e); }
		Element.hide(this.loadingEl);
		Element.hide(this.infoEl);
		this.infoEl.innerHTML = "";
		Element.show(this.containerEl);
	}, 
	
	refreshGroupList : function() {
        Element.wait(this.containerEl);
		var self = this;
		new Ajax.Updater(this.containerEl, habboReqPath + "/myhabbo/groups/grouplist", { method:"post", 
			parameters:"id="+encodeURIComponent(this.ownerId)+"&widgetId="+encodeURIComponent(this.widgetId), onComplete:function(req, json) {
				self.listEl = document.getElementsByClassName("groups-list", self.containerEl)[0];
				self._addOpenEventHandlers();
			} });
	}, 
	
	onCompleteCallback : function(req, json) {
		if (req.responseText == "OK") {
			hideGroupActionDialog(this);
			this.hideGroupDetails();
			this.refreshGroupList();
			return false;
		}
		return true;
	}, 
	
	_addOpenEventHandlers : function() {
		if (this.listEl) {
			this.listEl.onclick = this.showGroupDetails.bindAsEventListener(this);
		}
	}, 
	
	_parseGroupIdFromElementId : function(elementId) {
		return elementId.substring(elementId.lastIndexOf("-")+1);
	}
	
}


/**
 * Common add/remove tag controls for profile widget and group info widget.
 */
var TagWidgetPartial = Class.create();
TagWidgetPartial.prototype = {
    initialize : function(parentWidget) {
        this.parentWidget = parentWidget;
        TagHelper.init(this.parentWidget.loggedInAccountId);

        this.btnAddTag = $('profile-add-tag');
        if (this.btnAddTag) {
            this.eventAddTag = this.handleAddTag.bindAsEventListener(this);
            Event.observe(this.btnAddTag, "click", this.eventAddTag);
        }
        var self = this;
        Event.observe("profile-tag-list", "click", function(e) { self.handleClickTag(e);});

        if($('profile-add-tag-input')) {
        Event.observe($('profile-add-tag-input'), "keypress", function(e) {
                if (e.keyCode == Event.KEY_RETURN) {
                    self.handleAddTag(e);
                }
            });
        }

    },

    handleAddTag : function(e) {
        Event.stop(e);
        var tagName = $F('profile-add-tag-input');
        var tagField = $('profile-add-tag-input');
        if (this.parentWidget.tagType == "group") {
            this.addGroupTag(tagName, this.parentWidget.groupId);
        }
        else if (this.parentWidget.tagType == "avatar") {
            this.addAvatarTag(tagName, this.parentWidget.accountId);
        }
        tagField.value = "";
    },

    handleClickTag : function(e) {
        var element = Event.element(e);
        var tagName = element.className.substring(element.className.lastIndexOf("-") + 1);
        if (element.className.indexOf('tag-delete-link') >= 0) {
            Event.stop(e);
            this.errorMessage("valid"); // Clear the error box.

            if (this.parentWidget.tagType == "group") {
                new Ajax.Updater("profile-tag-list", "/myhabbo/tag/removegrouptag", {
                    parameters: "groupId=" + encodeURIComponent(this.parentWidget.groupId) + "&tagName=" + encodeURIComponent(tagName),
                    evalScripts : true
                });
            }
            else if (this.parentWidget.tagType == "avatar") {
                new Ajax.Updater("profile-tag-list", "/myhabbo/tag/remove", {
                    parameters: "accountId=" + encodeURIComponent(this.parentWidget.accountId) + "&tagName=" + encodeURIComponent(tagName),
                    evalScripts : true
                });
            }
        }
        else if (element.className.indexOf('tag-add-link') >= 0) {
            Event.stop(e);
            // The 'add this tag to my collection' icon was clicked.
            TagHelper.addThisTagToMe(tagName);
        }

    },


    /**
     * Displays the inline widget error box depending on the tag message code.
     */
    errorMessage : function(code) {
        var msgField = $('profile-tags-status-field');

        if (msgField) {
            var limitTagMsg = $('tag-limit-message');
            var invalidTagMsg = $('tag-invalid-message');
            if(code=="invalidtag") {
                msgField.style.display='block';
                invalidTagMsg.style.display='block';
                limitTagMsg.style.display='none';
            }
            else if(code=="taglimit") {
                msgField.style.display='block';
                limitTagMsg.style.display='block';
                invalidTagMsg.style.display='none';
            }
            else if(code=="valid") {
                msgField.style.display='none';
            }
        }
    },

    /**
     * Adds a tag to the account's tag collection.
     */
    addAvatarTag : function(tagName, accountId) {

        accountId = encodeURIComponent(accountId);
        var self = this;
        new Ajax.Request("/myhabbo/tag/add", {
            parameters: "accountId=" + accountId + "&tagName=" + encodeURIComponent(tagName),
            onSuccess: function(transport) {
                var tagMsgCode = transport.responseText;

                if(tagMsgCode=="valid" && $('profile-tags-status-field')) {
                    new Ajax.Updater("profile-tag-list", "/myhabbo/tag/list", {
                        parameters: "tagMsgCode=" + encodeURIComponent("valid") + "&accountId=" + accountId
                    });
                }

                self.errorMessage(tagMsgCode);

            }
        });
    },

    /**
     * Adds a tag to the group's tag collection.
     */
    addGroupTag : function(tagName, groupId) {

        groupId = encodeURIComponent(groupId);
        var self = this;
        new Ajax.Request("/myhabbo/tag/addgrouptag", {
            parameters: "groupId=" + groupId + "&tagName=" + encodeURIComponent(tagName),
            onSuccess: function(transport) {
                var tagMsgCode = transport.responseText;

                if(tagMsgCode=="valid" && $('profile-tags-status-field')) {
                    new Ajax.Updater("profile-tag-list", "/myhabbo/tag/listgrouptags", {
                        parameters: "tagMsgCode=" + encodeURIComponent("valid") + "&groupId=" + groupId
                    });
                }

                self.errorMessage(tagMsgCode);
            }
        });
    }

};


var ProfileWidget = Class.create();
ProfileWidget.prototype = {
    tagType : "avatar",
	initialize : function(accountId, loggedInAccountId) {
	    this.options = Object.extend({
	    	messageText: "Add as friend?",
	    	headerText: "Are you sure?"
	    }, arguments[2] || {});
	
		this.accountId = accountId;
        if (loggedInAccountId) {
			this.loggedInAccountId = loggedInAccountId;
		}
		this.btnAddFriend = $('add-friend');
		if (this.btnAddFriend) {
			this.eventAddFriend = this.handleAddFriend.bindAsEventListener(this);			
			Event.observe(this.btnAddFriend, "click", this.eventAddFriend);
		}

        this.tags = new TagWidgetPartial(this);

    },

    handleAddFriend : function(e) {
		if (Element.hasClassName(this.btnAddFriend, "disabled")) {
			Event.stop(e);
			return;
		}
		if (this.loggedInAccountId) {
			Event.stop(e);
			this.dialog = Dialog.showConfirmDialog(this.options.messageText, { okHandler : this.doSendFriendRequest.bind(this),
				headerText: this.options.headerText, buttonText: this.options.buttonText, cancelButtonText: this.options.cancelButtonText});
		}	
	},
	
	doSendFriendRequest : function() {
		Dialog.setAsWaitDialog(this.dialog);
		var self = this;
		new Ajax.Request("/myhabbo/friends/add", {
			parameters: "accountId=" + encodeURIComponent(this.accountId),
			onSuccess: function() {
				Element.addClassName(self.btnAddFriend, "disabled");
				Element.remove(self.dialog); 
				Overlay.hide();
			}
		});
	}
		
};


var GroupInfoWidget = Class.create();
GroupInfoWidget.prototype = {
    tagType : "group",
    initialize : function(groupId, loggedInAccountId) {

        this.groupId = groupId;
        if (loggedInAccountId) {
			this.loggedInAccountId = loggedInAccountId;
		}

        this.tags = new TagWidgetPartial(this);

    }

};


var HighscoreListWidget = Class.create();
Object.extend(HighscoreListWidget, {
  handleEditMenu: function(e) {
    if (chosenElement && chosenElement.id) {
      var widget = WidgetRegistry.getWidgetById(chosenElement.id);
      if (widget) {
        var menu = Event.element(e);
        widget.setGameId(menu.options[menu.selectedIndex].value);
      }
    }
  }
});
HighscoreListWidget.prototype = {
  
  initialize: function(widgetId, gameId, type) {
    this.widgetId = widgetId;
    this.gameId = gameId;
    this.type = type;
    this.period = null;
    
    this.scoresEl = $("highscorelist-scores-" + this.widgetId);
    if (this.scoresEl) { this.scoresEl.onclick = this.handleScoresClick.bindAsEventListener(this); }
  }, 
  
  selectGameId: function() {
    var self = this;
    var select = $("highscorelist-game");
    $A(select.options).each(function(option) {
      if (option.value == self.gameId) { select.selectedIndex = option.index; }
    });
  }, 
  
  handleScoresClick: function(e) {
    var el = Event.element(e);
    if (el.nodeName.toLowerCase() == "li" && el.id) {
      var type = this.type;
      var period = this.period;
      var page = -1;
      
      if (el.id.indexOf("highscorelist-period") != -1) {
        period = el.id.substring(("highscorelist-period-" + this.widgetId).length + 1);
      } else if (el.id.indexOf("highscorelist-type") != -1) {
        type = el.id.split("-").last();
        if (type == this.type) { return; }
      } else if (el.id.indexOf("highscorelist-page") != -1) {
        page = el.id.split("-").last();
      }
      
      this._loadScores(type, period, page);
    }
  }, 
  
  setGameId: function(gameId) {
    if (gameId != "" && gameId != this.gameId) {
      closeEditMenu();
      Element.wait(this.scoresEl);
      var self = this;
      new Ajax.Request(habboReqPath + "/myhabbo/highscorelist/setGameId", {
        method: "post", parameters: { widgetId: this.widgetId, gameId: gameId }, 
        onComplete: function(req) {
          self.scoresEl.innerHTML = req.responseText;
          self.gameId = gameId;
        }
      });
    }
  }, 
  
  _loadScores: function(type, period, page) {
    var self = this;
    var url = "/myhabbo/highscorelist/scores";
    if (page != -1) {
      url = "/myhabbo/highscorelist/page";
    } else {
      Element.wait(this.scoresEl);
    }
    
    new Ajax.Request(habboReqPath + url, {
      method: "post", parameters: {
        widgetId: this.widgetId, gameId: this.gameId, type: type, period: period, page: ((page != -1) ? page : 0)
      }, 
      onComplete: function(req) {
        if (page != -1) {
          $("highscorelist-page-" + self.widgetId).innerHTML = req.responseText;
        } else {
          self.scoresEl.innerHTML = req.responseText;
          self.type = type;
          self.period = period;
        }
      }
    });
  }
  
};
